/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/

#include <libc.h>
#include <stdio.h>

#include "oo7.h"
#include <QueryIterator.h>
#include <Directory.h>
#include "globals.h"
#include "GenParams.h"
#include "BenchParams.h"
#include "VarParams.h"


//////////////////////////////////////////////////////////////////
//
// Query #2 - 1% selection on AtomicParts via build date (the
// most recent 1% of AtomicParts.)  A B+ tree index is used.
//
// This is the SQL version.
//
/////////////////////////////////////////////////////////////////


extern int	debugMode;


//////////////////////////////////////////////////////////////////
//
// Routine to do Query #2
//
//////////////////////////////////////////////////////////////////


int query2()
{
    QueryIterator *qItr;
    AtomicPart *atomH;
    char theQuery[256];
    int count = 0;

    // create the iterator once to avoid overhead.
    qItr = new QueryIterator((char *) 0);

    // choose a range of dates with the appropriate selectivity
    int dateRange = (Query2Percent * (MaxAtomicDate - MinAtomicDate)) / 100;
    int lowerDate = MaxAtomicDate - dateRange;

    if (debugMode) {
        printf("    Running Query2SQL with dateRange = %d, lowerDate = %d\n",
		    dateRange, lowerDate);
    }

    sprintf(theQuery, 
                "select A from AllPartsById A where buildDate >= %d;",
                lowerDate);

printf("Trying query \"%s\".\n", theQuery);

    qItr->Reset(theQuery);
    if (qItr->SQLCode() == QueryFailed) {
        fprintf(stderr, "Query %s failed.\n", theQuery);
        OC_transactionAbort();
        exit(1);
    } else {
        while (qItr->moreData()) {
            // get next match and advance iterator.
            atomH = (AtomicPart*)(Entity*)(*qItr)();  
            if (debugMode) {
                printf("    In Query1, on AtomicPart = %d:\n", atomH->id);
            }
            // process part by calling the null procedure
            atomH->DoNothing();
            count++;
        }
    }
    delete qItr;
    return count;
}

/*  $Version: Query2SQL.C 1.1 93/02/19 16:07:20 Exp $ */

