/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/

#include <libc.h>
#include <stdio.h>
#include <List.h>

#include "oo7.h"
#include "globals.h"
#include "GenParams.h"
#include "BenchParams.h"
#include "VarParams.h"

//////////////////////////////////////////////////////////////////
//
// Query #4 - randomly choose "Query4RepeatCnt" document objects by 
// lookup on their title field.  An index can be used for the
// actual lookup.  
//
// "Hand-coded" version - no SQL.
//
/////////////////////////////////////////////////////////////////

extern int	debugMode;


static char	title[TitleSize];

//////////////////////////////////////////////////////////////////
//
// Routine to do Query #4
//
//////////////////////////////////////////////////////////////////


int  query4()
{
    Document*	      docH;
    ListIterator*     baseI;  // iterator for traversing subassemblies
    BaseAssembly*     ba;

    // now randomly select a document via its title index 
    // and trace a path to the base assemblies that use 
    // its associated composite part (and then
    // repeat this process the desired number of times)

    // set random seed so "hot" runs are truly hot
    srandom(1);

    int count = 0;
    for (int i = 0; i < Query4RepeatCnt; i++) 
    {
	// generate random document title and lookup document
	int compPartId = (int) (random() % TotalCompParts) + 1;
        sprintf(title, "Composite Part %08d", compPartId);
	docH = (Document*) (Entity*) 
                 AllDocumentsByTitle->getEntityElement(title);
	if (docH == NULL)
	{
            fprintf(stderr, "ERROR: Unable to find document called \"%s\"\n",
	            title);
	    OC_transactionAbort();
            exit(1);
        }

        if (debugMode) {
            printf("    In Query4, document title = \"%s\"\n", title);
        }

	// now walk back up the path(s) to the associated base assemblies
	// (based on private uses of the composite part, at least for now)

	baseI = new ListIterator(docH->part()->usedInPriv());  
		// establish baseassembly iterator
	while (baseI->moreData())
	{
            ba = (BaseAssembly*)(Entity*)(*baseI)();
	    ba->DoNothing();
	    count++;
	}
	delete baseI;
    }
    return count;
}

/*  $Version: Query4.C 1.1 93/02/19 16:07:25 Exp $ */

