/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/

#include <libc.h>
#include <stdio.h>
#include <List.h>

#include "oo7.h"
#include <QueryIterator.h>
#include <Directory.h>
#include "globals.h"
#include "GenParams.h"
#include "BenchParams.h"
#include "VarParams.h"


//////////////////////////////////////////////////////////////////
//
// Query #4 - randomly choose "Query4RepeatCnt" atomic parts by 
// lookup on their id field.  An index can be used for the
// actual lookup.  
//
// This is the SQL version.
//
/////////////////////////////////////////////////////////////////


extern int	debugMode;

//////////////////////////////////////////////////////////////////
//
// Routine to do Query #4SQL
//
//////////////////////////////////////////////////////////////////


int query4()
{
    QueryIterator *qItr;
    Document *docH;
    char theQuery[256];
    ListIterator *baseI;
    BaseAssembly *ba;
    int count = 0;
    char	title[TitleSize];

    // create the iterator once to avoid overhead.
    qItr = new QueryIterator((char *)0);

    // set random seed so "hot" runs are truly hot
    srandom(1);

    // now randomly select documents
    for (int i = 0; i < Query4RepeatCnt; i++) {

	// generate random document title and lookup document
	int compPartId = (int) (random() % TotalCompParts) + 1;
        sprintf(title, "Composite Part %08d", compPartId);
        sprintf(theQuery, 
          "select A from AllDocumentsByTitle A where title = \"%s\";", title);
printf("Running query %s\n", theQuery);

        qItr->Reset(theQuery);

printf("Reset the iterator.\n");
        if (qItr->SQLCode() == QueryFailed) {
            fprintf(stderr, "Query %s failed.\n", theQuery);
            OC_transactionAbort();
            exit(1);
        } else {
            while (qItr->moreData()) {
                // get next match and advance iterator.
                docH = (Document*)(Entity*)(*qItr)();  
	        if (docH == NULL) {
		    printf("Yup, docH was null!\n");
                }
                if (debugMode) {
                    printf("    In Query4, doc title = %s.\n", docH->title);
                }

	        // now walk back up to the associated base assemblies
	        // (private uses of the composite part, at least for now)

		// establish baseassembly iterator
	        baseI = new ListIterator(docH->part()->usedInPriv());  
		while (baseI->moreData()) {
		    ba = (BaseAssembly*)(Entity*)(*baseI)();
	            ba->DoNothing();
	            count++;
	        }
	        delete baseI;
            }
        }
    }
    delete qItr;
    return count;
}

/*  $Version: Query4SQL.C 1.1 93/02/19 16:07:26 Exp $ */

