/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/

#include <stdio.h>
#include "GenParams.h"

int NumAtomicPerComp;
int NumConnPerAtomic;
int DocumentSize;
int ManualSize;

int NumAssmPerAssm;
int NumCompPerAssm;

int NumCompPerModule;
int NumAssmLevels;
int TotalModules;

int TotalCompParts;
int TotalAtomicParts;

int CachePages;

//////////////////////////////////////////////////////////////////
// Set generation parameters for the benchmark.
//
// Assumes a configuration file "OO7.config" of the
// following format:
//
// NumAssmPerAssm     	n1
// NumCompPerAssm     	n2
// NumCompPerModule   	n3
// NumAssmLevels      	n4
// TotalModules       	n5
// NumAtomicPerComp	n6
// NumConnPerAtomic 	n7
// DocumentSize		n8
// ManualSize		n9
//
// where n1 through n0 are integers.  The order of
// parameters is critical!  (This is a dumb parser
// after all.) 
//////////////////////////////////////////////////////////////////

int SetParams(char* configFileName) {

	FILE	*configFile;

	configFile = fopen(configFileName, "r");
	if (!configFile) {
	  fprintf(stderr, "Couldn't open config file: %s\n",configFileName);
	  return(-1);
	}

	// Get parameters.
	fscanf(configFile, "%*s %d\n", &NumAssmPerAssm);
	printf("NumAssmPerAssm = %d.\n", NumAssmPerAssm); 

	fscanf(configFile, "%*s %d\n", &NumCompPerAssm); 
	printf("NumCompPerAssm = %d.\n", NumCompPerAssm); 

	fscanf(configFile, "%*s %d\n", &NumCompPerModule); 
	printf("NumCompPerModule = %d.\n", NumCompPerModule); 

	fscanf(configFile, "%*s %d\n", &NumAssmLevels); 
	printf("NumAssmLevels = %d.\n", NumAssmLevels); 

	fscanf(configFile, "%*s %d\n", &TotalModules); 
	printf("TotalModules = %d.\n", TotalModules); 

	fscanf(configFile, "%*s %d\n", &NumAtomicPerComp); 
	printf("NumAtomicPerComp = %d.\n", NumAtomicPerComp); 

	fscanf(configFile, "%*s %d\n", &NumConnPerAtomic); 
	printf("NumConnPerAtomic = %d.\n", NumConnPerAtomic); 

	fscanf(configFile, "%*s %d\n", &DocumentSize); 
	printf("DocumentSize = %d.\n", DocumentSize); 

	fscanf(configFile, "%*s %d\n", &ManualSize); 
	printf("ManualSize = %d.\n", ManualSize); 

	TotalCompParts = NumCompPerModule * TotalModules;
	printf("Setting TotalCompParts to %d.\n", TotalCompParts);

	TotalAtomicParts = TotalCompParts * NumAtomicPerComp;
	printf("Setting TotalAtomicParts to %d.\n", TotalAtomicParts);

	fscanf(configFile, "%*s %d\n", &CachePages); 
	printf("Cache Pages = %d.\n", CachePages); 

	return(1);
}

/*  $Version: SetParams.C 1.1 93/02/19 16:07:38 Exp $ */

