/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/

#include <libc.h>
#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include "oo7.h"
#include "globals.h"
#include "baidlist.h"

extern "C" int open(char*,int,mode_t);
extern "C" int close(int);

// Add a Base assembly reference to a list 

void BAIdList::insert(int id)
{
    BAListElem* elem = new BAListElem;
    if (elem == NULL) 
    {
    	printf("new failed in BAIdlist.insert()\n");
	return;
    }
    elem->baId = id;
    elem->next = listPtr;
    listPtr = elem;
};

// initialize a scan
void BAIdList::initScan()
{
    current = listPtr;
};

// Scan 
BaseAssembly* BAIdList::next()
{
    int id;
    BaseAssembly* ba;

    if (current == NULL) return NULL;
    else
    {
	id = current->baId;
	ba = (BaseAssembly*)(Entity*) AllBaseAssemblies->getEntityElement(id);
	if (ba == NULL) {
            fprintf(stderr, "ERROR: Unable to find base assembly %d\n", id);
            OC_transactionAbort();
            exit(1);
        }

	current = current->next;
	// take the id and look up the object
	return ba;
    }
};

// save an array of baidlists of size size into fileName
int BAIdList::save(int size, char *fileName)
{
	int 	i;
	int 	cnt;
	int	wcnt;
	int	fileId;
	BAListElem* current;

	fileId = open(fileName, O_CREAT | O_RDWR, 0666);
	if (fileId < 0) {
	    fprintf(stderr, "Couldn't open baidlist file: %s\n", fileName);
	    return (-1);
	}
	for (i=0;i<size;i++)
	{
// 	    printf("element %d\t",i);
	    wcnt = write(fileId, (char*) &i, sizeof(int));
	    if (wcnt < sizeof(int)) printf("write error, wcnt=%d\n",wcnt);
	    // scan list counting number of elements
	    cnt = 0;
	    current = this[i].listPtr;
	    while (current != NULL)
	    {
		cnt++;
//		printf("baId=%d, ",current->baId);
		current = current->next;
	    }
	    wcnt = write(fileId, (char*) &cnt, sizeof(int));
	    if (wcnt < sizeof(int)) printf("write error, wcnt=%d\n",wcnt);
	    current = this[i].listPtr;
	    while (current != NULL)
	    {
	        wcnt = write(fileId, (char*) &current->baId, sizeof(int));
	        if (wcnt < sizeof(int)) printf("write error, wcnt=%d\n",wcnt);
		current = current->next;
	    }
//	    printf("\t, cnt=%d\n",cnt);
	}
	close(fileId);
	return(0);
};


// restore baidlists from file
int BAIdList::restore(int size, char* fileName)
{
	int 	i,j;
	int 	cnt, rcnt;
	int	fileId;
	int	id, baId;
	BAIdList* current;

//	printf("restore called. size=%d, file=%s\n",size,fileName);
	fileId = open(fileName, O_RDWR,00);
	if (fileId < 0) {
	    printf("Couldn't open baidlist file: %s\n", fileName);
	    return (-1);
	}
//	printf("file opened, fileId = %d\n",fileId);
	current = this;
	for (i=0;i < size; i++)
	{

	    rcnt = read(fileId, (char*) &id, sizeof(int));
	    if (rcnt < sizeof(int)) printf("read error, rcnt=%d\n",rcnt);
	    if (id != i) printf("expected %d got %d\n",i,id);
	    rcnt = read(fileId, (char*) &cnt, sizeof(int));
	    if (rcnt < sizeof(int)) printf("read error, rcnt=%d\n",rcnt);
//	    printf("id=%d, cnt=%d,\t",id,cnt);
	    for (j=0;j<cnt;j++)
	    {
	        rcnt = read(fileId, (char*) &baId, sizeof(int));
	        if (rcnt < sizeof(int)) printf("read error, rcnt=%d\n",rcnt);
		current->insert(baId);
//		printf("baId=%d, ",baId);
	    }
	    current++;
//	    printf("\n");
	}
	close(fileId);
	return(0);
};
